@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -333,13 +333,13 @@ static MagickBooleanType CompositeOverImage(Image *image,
       canvas_pixel,
       source_pixel;
 
-    register const Quantum
+    const Quantum
       *magick_restrict p;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       x;
 
     if (status == MagickFalse)
@@ -391,7 +391,7 @@ static MagickBooleanType CompositeOverImage(Image *image,
         Sc,
         Sca;
 
-      register ssize_t
+      ssize_t
         i;
 
       size_t
@@ -624,13 +624,13 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
         MagickBooleanType
           sync;
 
-        register const Quantum
+        const Quantum
           *p;
 
-        register Quantum
+        Quantum
           *q;
 
-        register ssize_t
+        ssize_t
           x;
 
         if (status == MagickFalse)
@@ -646,7 +646,7 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
           }
         for (x=0; x < (ssize_t) source_image->columns; x++)
         {
-          register ssize_t
+          ssize_t
             i;
 
           if (GetPixelReadMask(source_image,p) <= (QuantumRange/2))
@@ -708,13 +708,13 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
         MagickBooleanType
           sync;
 
-        register const Quantum
+        const Quantum
           *p;
 
-        register Quantum
+        Quantum
           *q;
 
-        register ssize_t
+        ssize_t
           x;
 
         if (status == MagickFalse)
@@ -777,7 +777,7 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
       CacheView
         *canvas_view;
 
-      MagickRealType
+      double
         angle_range,
         angle_start,
         height,
@@ -793,13 +793,11 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
         blur;
 
       /*
-        Blur Image by resampling.
-
-        Blur Image dictated by an overlay gradient map: X = red_channel;
-          Y = green_channel; compose:args =  x_scale[,y_scale[,angle]].
+        Blur Image by resampling dictated by an overlay gradient map:
+          X = red_channel; Y = green_channel; compose:args =
+          x_scale[,y_scale[,angle]].
       */
-      canvas_image=CloneImage(image,0,0,MagickTrue,
-        exception);
+      canvas_image=CloneImage(image,0,0,MagickTrue,exception);
       if (canvas_image == (Image *) NULL)
         {
           source_image=DestroyImage(source_image);
@@ -822,12 +820,13 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
         }
       /*
         Users input sigma now needs to be converted to the EWA ellipse size.
-        The filter defaults to a sigma of 0.5 so to make this match the
-        users input the ellipse size needs to be doubled.
+        The filter defaults to a sigma of 0.5 so to make this match the users
+        input the ellipse size needs to be doubled.
       */
-      width=height=geometry_info.rho*2.0;
-      if ((flags & HeightValue) != 0 )
-        height=geometry_info.sigma*2.0;
+      width=2.0*geometry_info.rho;
+      height=width;
+      if ((flags & HeightValue) != 0)
+        height=2.0*geometry_info.sigma;
       /*
         Default the unrotated ellipse width and height axis vectors.
       */
@@ -835,23 +834,27 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
       blur.x2=0.0;
       blur.y1=0.0;
       blur.y2=height;
-      /* rotate vectors if a rotation angle is given */
       if ((flags & XValue) != 0 )
         {
           MagickRealType
             angle;
 
+          /*
+            Rotate vectors if a rotation angle is given.
+          */
           angle=DegreesToRadians(geometry_info.xi);
           blur.x1=width*cos(angle);
           blur.x2=width*sin(angle);
           blur.y1=(-height*sin(angle));
           blur.y2=height*cos(angle);
         }
-      /* Otherwise lets set a angle range and calculate in the loop */
       angle_start=0.0;
       angle_range=0.0;
       if ((flags & YValue) != 0 )
         {
+          /*
+            Lets set a angle range and calculate in the loop.
+          */
           angle_start=DegreesToRadians(geometry_info.xi);
           angle_range=DegreesToRadians(geometry_info.psi)-angle_start;
         }
@@ -860,16 +863,15 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
 
         As the minimum ellipse radius of support*1.0 the EWA algorithm
         can only produce a minimum blur of 0.5 for Gaussian (support=2.0)
-        This means that even 'No Blur' will be still a little blurry!
-
-        The solution (as well as the problem of preventing any user
-        expert filter settings, is to set our own user settings, then
-        restore them afterwards.
+        This means that even 'No Blur' will be still a little blurry! The
+        solution (as well as the problem of preventing any user expert filter
+        settings, is to set our own user settings, restore them afterwards.
       */
       resample_filter=AcquireResampleFilter(image,exception);
       SetResampleFilter(resample_filter,GaussianFilter);
-
-      /* do the variable blurring of each pixel in image */
+      /*
+        Perform the variable blurring of each pixel in image.
+      */
       GetPixelInfo(image,&pixel);
       source_view=AcquireVirtualCacheView(source_image,exception);
       canvas_view=AcquireAuthenticCacheView(canvas_image,exception);
@@ -878,13 +880,13 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
         MagickBooleanType
           sync;
 
-        register const Quantum
+        const Quantum
           *magick_restrict p;
 
-        register Quantum
+        Quantum
           *magick_restrict q;
 
-        register ssize_t
+        ssize_t
           x;
 
         if (((y+y_offset) < 0) || ((y+y_offset) >= (ssize_t) image->rows))
@@ -902,7 +904,7 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
               p+=GetPixelChannels(source_image);
               continue;
             }
-          if (fabs((double) angle_range) > MagickEpsilon)
+          if (fabs(angle_range) > MagickEpsilon)
             {
               MagickRealType
                 angle;
@@ -1049,13 +1051,13 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
         MagickBooleanType
           sync;
 
-        register const Quantum
+        const Quantum
           *magick_restrict p;
 
-        register Quantum
+        Quantum
           *magick_restrict q;
 
-        register ssize_t
+        ssize_t
           x;
 
         if (((y+y_offset) < 0) || ((y+y_offset) >= (ssize_t) image->rows))
@@ -1229,13 +1231,13 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
       canvas_pixel,
       source_pixel;
 
-    register const Quantum
+    const Quantum
       *magick_restrict p;
 
-    register Quantum
+    Quantum
       *magick_restrict q;
 
-    register ssize_t
+    ssize_t
       x;
 
     if (status == MagickFalse)
@@ -1292,7 +1294,7 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
         Sc,
         Sca;
 
-      register ssize_t
+      ssize_t
         i;
 
       size_t
@@ -1506,6 +1508,7 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
         case HueCompositeOp:
         case LuminizeCompositeOp:
         case ModulateCompositeOp:
+        case RMSECompositeOp:
         case SaturateCompositeOp:
         {
           GetPixelInfoPixel(source_image,p,&source_pixel);
@@ -1579,6 +1582,7 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
               case ColorizeCompositeOp:
               case HueCompositeOp:
               case LuminizeCompositeOp:
+              case RMSECompositeOp:
               case SaturateCompositeOp:
               {
                 if (fabs((double) (QuantumRange*Sa-TransparentAlpha)) < MagickEpsilon)
@@ -1607,6 +1611,7 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
                   pixel=QuantumRange*Sa;
                 break;
               }
+              case BlurCompositeOp:
               case CopyCompositeOp:
               case DisplaceCompositeOp:
               case DistortCompositeOp:
@@ -1625,7 +1630,7 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
               }
               case DifferenceCompositeOp:
               {
-                pixel=QuantumRange*fabs(Sa-Da);
+                pixel=QuantumRange*fabs((double) (Sa-Da));
                 break;
               }
               case FreezeCompositeOp:
@@ -1749,7 +1754,6 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
             pixel=gamma*(source_dissolve*Sa*Sc+canvas_dissolve*Da*Dc);
             break;
           }
-          case BlurCompositeOp:
           case CopyCompositeOp:
           case ReplaceCompositeOp:
           case SrcCompositeOp:
@@ -1757,6 +1761,7 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
             pixel=QuantumRange*Sca;
             break;
           }
+          case BlurCompositeOp:
           case DisplaceCompositeOp:
           case DistortCompositeOp:
           {
@@ -2276,6 +2281,37 @@ MagickExport MagickBooleanType CompositeImage(Image *image,
               pixel=QuantumRange;
             break;
           }
+          case RMSECompositeOp:
+          {
+            double
+              gray;
+
+            if (fabs((double) (QuantumRange*Sa-TransparentAlpha)) < MagickEpsilon)
+              {
+                pixel=Dc;
+                break;
+              }
+            if (fabs((double) (QuantumRange*Da-TransparentAlpha)) < MagickEpsilon)
+              {
+                pixel=Sc;
+                break;
+              }
+            gray=sqrt(
+              (canvas_pixel.red-source_pixel.red)*
+              (canvas_pixel.red-source_pixel.red)+
+              (canvas_pixel.green-source_pixel.green)*
+              (canvas_pixel.green-source_pixel.green)+
+              (canvas_pixel.blue-source_pixel.blue)*
+              (canvas_pixel.blue-source_pixel.blue)/3.0);
+            switch (channel)
+            {
+              case RedPixelChannel: pixel=gray; break;
+              case GreenPixelChannel: pixel=gray; break;
+              case BluePixelChannel: pixel=gray; break;
+              default: pixel=Dc; break;
+            }
+            break;
+          }
           case SaturateCompositeOp:
           {
             if (fabs((double) (QuantumRange*Sa-TransparentAlpha)) < MagickEpsilon)
@@ -2511,7 +2547,7 @@ MagickExport MagickBooleanType TextureImage(Image *image,const Image *texture,
       */
       for (y=0; y < (ssize_t) image->rows; y+=(ssize_t) texture_image->rows)
       {
-        register ssize_t
+        ssize_t
           x;
 
         if (status == MagickFalse)
@@ -2561,14 +2597,14 @@ MagickExport MagickBooleanType TextureImage(Image *image,const Image *texture,
     MagickBooleanType
       sync;
 
-    register const Quantum
+    const Quantum
       *p,
       *pixels;
 
-    register ssize_t
+    ssize_t
       x;
 
-    register Quantum
+    Quantum
       *q;
 
     size_t
@@ -2587,7 +2623,7 @@ MagickExport MagickBooleanType TextureImage(Image *image,const Image *texture,
       }
     for (x=0; x < (ssize_t) image->columns; x+=(ssize_t) texture_image->columns)
     {
-      register ssize_t
+      ssize_t
         j;
 
       p=pixels;
@@ -2596,7 +2632,7 @@ MagickExport MagickBooleanType TextureImage(Image *image,const Image *texture,
         width=image->columns-x;
       for (j=0; j < (ssize_t) width; j++)
       {
-        register ssize_t
+        ssize_t
           i;
 
         for (i=0; i < (ssize_t) GetPixelChannels(texture_image); i++)
