@@ -17,7 +17,7 @@
 %                                 August 1996                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -675,7 +675,7 @@ MagickPrivate void ConvertHWBToRGB(const double hue,const double whiteness,
     r,
     v;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -692,7 +692,7 @@ MagickPrivate void ConvertHWBToRGB(const double hue,const double whiteness,
       *blue=QuantumRange*v;
       return;
     }
-  i=(ssize_t) floor(6.0*hue);
+  i=CastDoubleToLong(floor(6.0*hue));
   f=6.0*hue-i;
   if ((i & 0x01) != 0)
     f=1.0-f;
@@ -1578,7 +1578,7 @@ MagickPrivate double GenerateDifferentialNoise(RandomInfo *random_info,
       double
         poisson;
 
-      register ssize_t
+      ssize_t
         i;
 
       poisson=exp(-SigmaPoisson*QuantumScale*pixel);
@@ -1587,7 +1587,7 @@ MagickPrivate double GenerateDifferentialNoise(RandomInfo *random_info,
         beta=GetPseudoRandomValue(random_info);
         alpha*=beta;
       }
-      noise=(double) (QuantumRange*i/SigmaPoisson);
+      noise=(double) (QuantumRange*i*PerceptibleReciprocal(SigmaPoisson));
       break;
     }
     case RandomNoise:
@@ -1640,7 +1640,7 @@ MagickPrivate size_t GetOptimalKernelWidth1D(const double radius,
     normalize,
     value;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
