@@ -18,7 +18,7 @@
 %                             November 1998                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -296,7 +296,7 @@ MagickExport MagickBooleanType GetImageMagick(const unsigned char *magick,
   MagickBooleanType
     status;
 
-  register const MagickInfo
+  const MagickInfo
     *p;
 
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
@@ -610,7 +610,7 @@ MagickExport MagickBooleanType GetMagickEndianSupport(
 MagickExport const MagickInfo *GetMagickInfo(const char *name,
   ExceptionInfo *exception)
 {
-  register const MagickInfo
+  const MagickInfo
     *magick_info;
 
   /*
@@ -703,10 +703,10 @@ MagickExport const MagickInfo **GetMagickInfoList(const char *pattern,
   const MagickInfo
     **formats;
 
-  register const MagickInfo
+  const MagickInfo
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -777,7 +777,7 @@ extern "C" {
 
 static int MagickCompare(const void *x,const void *y)
 {
-  register const char
+  const char
     **p,
     **q;
 
@@ -796,10 +796,10 @@ MagickExport char **GetMagickList(const char *pattern,
   char
     **formats;
 
-  register const MagickInfo
+  const MagickInfo
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -1067,7 +1067,7 @@ MagickExport MagickBooleanType GetMagickUseExtension(
 
 static void *DestroyMagickNode(void *magick_info)
 {
-  register MagickInfo
+  MagickInfo
     *p;
 
   p=(MagickInfo *) magick_info;
@@ -1177,7 +1177,7 @@ MagickExport MagickBooleanType ListMagickInfo(FILE *file,
   const MagickInfo
     **magick_info;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
@@ -1393,6 +1393,7 @@ static SignalHandler *SetMagickSignalHandler(int signal_number,
 #if defined(SA_ONSTACK)
   action.sa_flags|=SA_ONSTACK;
 #endif
+  previous_action.sa_handler=SIG_DFL;
   status=sigaction(signal_number,&action,&previous_action);
   if (status < 0)
     return(SIG_ERR);
@@ -1802,7 +1803,7 @@ MagickExport int SetMagickPrecision(const int precision)
 */
 MagickExport MagickBooleanType UnregisterMagickInfo(const char *name)
 {
-  register const MagickInfo
+  const MagickInfo
     *p;
 
   MagickBooleanType
