@@ -18,7 +18,7 @@
 %                               December 2001                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -206,11 +206,11 @@ static MagickBooleanType DecodeImage(Image *image,const size_t compression,
     byte,
     count;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
-  register unsigned char
+  unsigned char
     *p,
     *q;
 
@@ -383,14 +383,14 @@ static size_t EncodeImage(Image *image,const size_t bytes_per_line,
   MagickBooleanType
     status;
 
-  register const unsigned char
+  const unsigned char
     *p;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
-  register unsigned char
+  unsigned char
     *q;
 
   ssize_t
@@ -533,14 +533,14 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Quantum
     index;
 
-  register Quantum
+  Quantum
     *q;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
-  register unsigned char
+  unsigned char
     *p;
 
   size_t
@@ -1060,7 +1060,7 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
     (void) memset(&quantum_bits,0,sizeof(quantum_bits));
     if ((bmp_info.bits_per_pixel == 16) || (bmp_info.bits_per_pixel == 32))
       {
-        register unsigned int
+        unsigned int
           sample;
 
         /*
@@ -1664,14 +1664,14 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image,
   MemoryInfo
     *pixel_info;
 
-  register const Quantum
+  const Quantum
     *p;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
-  register unsigned char
+  unsigned char
     *q;
 
   size_t
@@ -1705,8 +1705,8 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image,
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
-  if (((image->columns << 3) != (int) (image->columns << 3)) ||
-      ((image->rows << 3) != (int) (image->rows << 3)))
+  if (((image->columns << 3) != (size_t) ((int) (image->columns << 3))) ||
+      ((image->rows << 3) != (size_t) ((int) (image->rows << 3))))
     ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
   type=4;
   if (LocaleCompare(image_info->magick,"BMP2") == 0)
@@ -2032,7 +2032,7 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image,
           q=pixels+(image->rows-y-1)*bytes_per_line;
           for (x=0; x < (ssize_t) image->columns; x++)
           {
-            *q++=(unsigned char) GetPixelIndex(image,p);
+            *q++=(unsigned char) ((ssize_t) GetPixelIndex(image,p));
             p+=GetPixelChannels(image);
           }
           for ( ; x < (ssize_t) bytes_per_line; x++)
