@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -179,7 +179,7 @@ static MagickOffsetType GetFITSPixelExtrema(Image *image,
   MagickSizeType
     number_pixels;
 
-  register MagickOffsetType
+  MagickOffsetType
     i;
 
   offset=TellBlob(image);
@@ -207,7 +207,7 @@ static inline double GetFITSPixelRange(const size_t depth)
 static void SetFITSUnsignedPixels(const size_t length,
   const size_t bits_per_pixel,const EndianType endian,unsigned char *pixels)
 {
-  register ssize_t
+  ssize_t
     i;
 
   if (endian != MSBEndian)
@@ -270,11 +270,11 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   MagickSizeType
     number_pixels;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
-  register Quantum
+  Quantum
     *q;
 
   ssize_t
@@ -321,7 +321,7 @@ static Image *ReadFITSImage(const ImageInfo *image_info,
   {
     for ( ; EOFBlob(image) == MagickFalse; )
     {
-      register char
+      char
         *p;
 
       count=ReadBlob(image,8,(unsigned char *) keyword);
@@ -625,8 +625,8 @@ static inline void CopyFitsRecord(char *buffer,const char *data,
     return;
   length=MagickMin(strlen(data),80);
   if (length > (size_t) (FITSBlocksize-offset))
-    length=FITSBlocksize-offset-1;
-  (void) CopyMagickString(buffer+offset,data,length+1);
+    length=FITSBlocksize-offset;
+  (void) strncpy(buffer+offset,data,length);
 }
 
 static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
@@ -634,8 +634,7 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
 {
   char
     *fits_info,
-    header[FITSBlocksize],
-    *url;
+    header[FITSBlocksize];
 
   MagickBooleanType
     status;
@@ -643,7 +642,7 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
   QuantumInfo
     *quantum_info;
 
-  register const Quantum
+  const Quantum
     *p;
 
   size_t
@@ -739,12 +738,11 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
       CopyFitsRecord(fits_info,header,offset);
       offset+=80;
     }
-  url=GetMagickHomeURL();
-  (void) FormatLocaleString(header,FITSBlocksize,"HISTORY %.72s",url);
-  url=DestroyString(url);
+  (void) FormatLocaleString(header,FITSBlocksize,"HISTORY %.72s",
+    MagickAuthoritativeURL);
   CopyFitsRecord(fits_info,header,offset);
   offset+=80;
-  (void) CopyMagickString(header,"END",FITSBlocksize);
+  (void) strncpy(header,"END",FITSBlocksize);
   CopyFitsRecord(fits_info,header,offset);
   offset+=80;
   (void) WriteBlob(image,FITSBlocksize,(unsigned char *) fits_info);
