@@ -2,24 +2,18 @@
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
+%                             IIIII  PPPP   L                                 %
+%                               I    P   P  L                                 %
+%                               I    PPPP   L                                 %
+%                               I    P      L                                 %
+%                             IIIII  P      LLLLL                             %
 %                                                                             %
-%                     IIIIIIIIII    PPPPPPPP      LL                          %
-%                         II        PP      PP    LL                          %
-%                         II        PP       PP   LL                          %
-%                         II        PP      PP    LL                          %
-%                         II        PPPPPPPP      LL                          %
-%                         II        PP            LL                          %
-%                         II        PP            LL                          %
-%                     IIIIIIIIII    PP            LLLLLLLL                    %
+%                 Read/Write Scanalytics IPLab Image Format                   %
+%                                Sean Burke                                   %
+%                                2008.05.07                                   %
+%                                   v 0.9                                     %
 %                                                                             %
-%                                                                             %
-%                                                                             %
-%                   Read/Write Scanalytics IPLab Image Format                 %
-%                                  Sean Burke                                 %
-%                                  2008.05.07                                 %
-%                                     v 0.9                                   %
-%                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -181,7 +175,7 @@ static Image *ReadIPLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   Image *image;
 
   MagickBooleanType status;
-  register Quantum *q;
+  Quantum *q;
   unsigned char magick[12], *pixels;
   ssize_t count;
   ssize_t y;
@@ -521,7 +515,7 @@ static MagickBooleanType WriteIPLImage(const ImageInfo *image_info,Image *image,
   MagickOffsetType
     scene;
   
-  register const Quantum
+  const Quantum
     *p;
 
   QuantumInfo
