@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2015 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -39,31 +39,33 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/attribute.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/cache.h"
-#include "magick/color-private.h"
-#include "magick/colormap.h"
-#include "magick/colormap-private.h"
-#include "magick/colorspace.h"
-#include "magick/colorspace-private.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/monitor.h"
-#include "magick/monitor-private.h"
-#include "magick/pixel-accessor.h"
-#include "magick/property.h"
-#include "magick/quantum-private.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/module.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/artifact.h"
+#include "MagickCore/attribute.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/cache.h"
+#include "MagickCore/color-private.h"
+#include "MagickCore/colormap.h"
+#include "MagickCore/colormap-private.h"
+#include "MagickCore/colorspace.h"
+#include "MagickCore/colorspace-private.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/monitor.h"
+#include "MagickCore/monitor-private.h"
+#include "MagickCore/option.h"
+#include "MagickCore/pixel-accessor.h"
+#include "MagickCore/property.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/module.h"
 
 /*
   Enumerated declaractions.
@@ -112,7 +114,7 @@ typedef struct _TGAInfo
   Forward declarations.
 */
 static MagickBooleanType
-  WriteTGAImage(const ImageInfo *,Image *);
+  WriteTGAImage(const ImageInfo *,Image *,ExceptionInfo *);
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -140,40 +142,35 @@ static MagickBooleanType
 %    o exception: return any errors or warnings in this structure.
 %
 */
-static Image *ReadTGAImage(const ImageInfo *image_info,
-  ExceptionInfo *exception)
+static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   Image
     *image;
 
-  IndexPacket
-    index;
-
   MagickBooleanType
     status;
 
-  PixelPacket
+  PixelInfo
     pixel;
 
-  register IndexPacket
-    *indexes;
+  Quantum
+    index;
 
-  register PixelPacket
+  Quantum
     *q;
 
-  register ssize_t
+  ssize_t
     i,
     x;
 
   size_t
     base,
     flag,
-    offset,
-    real,
     skip;
 
   ssize_t
     count,
+    offset,
     y;
 
   TGAInfo
@@ -192,13 +189,13 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  image=AcquireImage(image_info);
+  assert(exception->signature == MagickCoreSignature);
+  image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -241,25 +238,29 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
   */
   image->columns=tga_info.width;
   image->rows=tga_info.height;
-  alpha_bits=(tga_info.attributes & 0x0FU);
-  image->matte=(alpha_bits > 0) || (tga_info.bits_per_pixel == 32) ||
-    (tga_info.colormap_size == 32) ?  MagickTrue : MagickFalse;
+  if ((tga_info.image_type != TGAMonochrome) &&
+      (tga_info.image_type != TGARLEMonochrome))
+    {
+      alpha_bits=(tga_info.attributes & 0x0FU);
+      image->alpha_trait=(alpha_bits > 0) || (tga_info.bits_per_pixel == 32) ||
+        (tga_info.colormap_size == 32) ?  BlendPixelTrait : UndefinedPixelTrait;
+    }
   if ((tga_info.image_type != TGAColormap) &&
       (tga_info.image_type != TGARLEColormap))
     image->depth=(size_t) ((tga_info.bits_per_pixel <= 8) ? 8 :
-      (tga_info.bits_per_pixel <= 16) ? 5 :
-      (tga_info.bits_per_pixel == 24) ? 8 :
-      (tga_info.bits_per_pixel == 32) ? 8 : 8);
+      (tga_info.bits_per_pixel <= 16) ? 5 : 8);
   else
     image->depth=(size_t) ((tga_info.colormap_size <= 8) ? 8 :
-      (tga_info.colormap_size <= 16) ? 5 :
-      (tga_info.colormap_size == 24) ? 8 :
-      (tga_info.colormap_size == 32) ? 8 : 8);
+      (tga_info.colormap_size <= 16) ? 5 : 8);
   if ((tga_info.image_type == TGAColormap) ||
-      (tga_info.image_type == TGAMonochrome) ||
-      (tga_info.image_type == TGARLEColormap) ||
-      (tga_info.image_type == TGARLEMonochrome))
+      (tga_info.image_type == TGARLEColormap))
     image->storage_class=PseudoClass;
+  if ((tga_info.image_type == TGAMonochrome) ||
+      (tga_info.image_type == TGARLEMonochrome))
+    {
+      image->type=GrayscaleType;
+      image->colorspace=GRAYColorspace;
+    }
   image->compression=NoCompression;
   if ((tga_info.image_type == TGARLEColormap) ||
       (tga_info.image_type == TGARLEMonochrome) ||
@@ -276,7 +277,10 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
 
           one=1;
           image->colors=one << tga_info.bits_per_pixel;
-          if (AcquireImageColormap(image,image->colors) == MagickFalse)
+          if ((MagickSizeType) image->colors > GetBlobSize(image))
+            ThrowReaderException(CorruptImageError,
+              "InsufficientImageDataInFile");
+          if (AcquireImageColormap(image,image->colors,exception) == MagickFalse)
             ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
         }
     }
@@ -293,35 +297,52 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
       */
       length=(size_t) tga_info.id_length;
       comment=(char *) NULL;
-      if (~length >= (MaxTextExtent-1))
-        comment=(char *) AcquireQuantumMemory(length+MaxTextExtent,
+      if (~length >= (MagickPathExtent-1))
+        comment=(char *) AcquireQuantumMemory(length+MagickPathExtent,
           sizeof(*comment));
       if (comment == (char *) NULL)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-      count=ReadBlob(image,tga_info.id_length,(unsigned char *) comment);
-      comment[tga_info.id_length]='\0';
-      (void) SetImageProperty(image,"comment",comment);
+      count=ReadBlob(image,length,(unsigned char *) comment);
+      if (count == (ssize_t) length)
+        {
+          comment[length]='\0';
+          (void) SetImageProperty(image,"comment",comment,exception);
+        }
       comment=DestroyString(comment);
     }
+  image->orientation=BottomLeftOrientation;
+  if ((tga_info.attributes & (1UL << 4)) == 0)
+    {
+      if ((tga_info.attributes & (1UL << 5)) == 0)
+        image->orientation=BottomLeftOrientation;
+      else
+        image->orientation=TopLeftOrientation;
+    }
+  else
+    {
+      if ((tga_info.attributes & (1UL << 5)) == 0)
+        image->orientation=BottomRightOrientation;
+      else
+        image->orientation=TopRightOrientation;
+    }
   if (image_info->ping != MagickFalse)
     {
       (void) CloseBlob(image);
       return(image);
     }
-  status=SetImageExtent(image,image->columns,image->rows);
+  status=SetImageExtent(image,image->columns,image->rows,exception);
   if (status == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
-  (void) ResetMagickMemory(&pixel,0,sizeof(pixel));
-  pixel.opacity=(Quantum) OpaqueOpacity;
+    return(DestroyImageList(image));
+  (void) memset(&pixel,0,sizeof(pixel));
+  pixel.alpha=(MagickRealType) OpaqueAlpha;
   if (tga_info.colormap_type != 0)
     {
       /*
         Read TGA raster colormap.
       */
-      if (AcquireImageColormap(image,image->colors) == MagickFalse)
+      if (image->colors < tga_info.colormap_index)
+        image->colors=tga_info.colormap_index;
+      if (AcquireImageColormap(image,image->colors,exception) == MagickFalse)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
       for (i=0; i < (ssize_t) tga_info.colormap_index; i++)
         image->colormap[i]=pixel;
@@ -335,7 +356,8 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
             /*
               Gray scale.
             */
-            pixel.red=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
+            pixel.red=(MagickRealType) ScaleCharToQuantum((unsigned char)
+              ReadBlobByte(image));
             pixel.green=pixel.red;
             pixel.blue=pixel.red;
             break;
@@ -352,10 +374,11 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
             j=(unsigned char) ReadBlobByte(image);
             k=(unsigned char) ReadBlobByte(image);
             range=GetQuantumRange(5UL);
-            pixel.red=ScaleAnyToQuantum(1UL*(k & 0x7c) >> 2,range);
-            pixel.green=ScaleAnyToQuantum((1UL*(k & 0x03) << 3)+
-              (1UL*(j & 0xe0) >> 5),range);
-            pixel.blue=ScaleAnyToQuantum(1UL*(j & 0x1f),range);
+            pixel.red=(MagickRealType) ScaleAnyToQuantum(1UL*(k & 0x7c) >> 2,
+              range);
+            pixel.green=(MagickRealType) ScaleAnyToQuantum((1UL*(k & 0x03)
+              << 3)+(1UL*(j & 0xe0) >> 5),range);
+            pixel.blue=(MagickRealType) ScaleAnyToQuantum(1UL*(j & 0x1f),range);
             break;
           }
           case 24:
@@ -363,9 +386,12 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
             /*
               8 bits each of blue, green and red.
             */
-            pixel.blue=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
-            pixel.green=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
-            pixel.red=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
+            pixel.blue=(MagickRealType) ScaleCharToQuantum((unsigned char)
+              ReadBlobByte(image));
+            pixel.green=(MagickRealType) ScaleCharToQuantum((unsigned char)
+              ReadBlobByte(image));
+            pixel.red=(MagickRealType) ScaleCharToQuantum((unsigned char)
+              ReadBlobByte(image));
             break;
           }
           case 32:
@@ -373,11 +399,14 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
             /*
               8 bits each of blue, green, red, and alpha.
             */
-            pixel.blue=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
-            pixel.green=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
-            pixel.red=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
-            pixel.opacity=(Quantum) (QuantumRange-ScaleCharToQuantum(
-              (unsigned char) ReadBlobByte(image)));
+            pixel.blue=(MagickRealType) ScaleCharToQuantum((unsigned char)
+              ReadBlobByte(image));
+            pixel.green=(MagickRealType) ScaleCharToQuantum((unsigned char)
+              ReadBlobByte(image));
+            pixel.red=(MagickRealType) ScaleCharToQuantum((unsigned char)
+              ReadBlobByte(image));
+            pixel.alpha=(MagickRealType) ScaleCharToQuantum((unsigned char)
+              ReadBlobByte(image));
             break;
           }
         }
@@ -390,19 +419,14 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
   base=0;
   flag=0;
   skip=MagickFalse;
-  real=0;
-  index=(IndexPacket) 0;
+  index=0;
   runlength=0;
   offset=0;
   for (y=0; y < (ssize_t) image->rows; y++)
   {
-    real=offset;
-    if (((unsigned char) (tga_info.attributes & 0x20) >> 5) == 0)
-      real=image->rows-real-1;
-    q=QueueAuthenticPixels(image,0,(ssize_t) real,image->columns,1,exception);
-    if (q == (PixelPacket *) NULL)
+    q=QueueAuthenticPixels(image,0,offset,image->columns,1,exception);
+    if (q == (Quantum *) NULL)
       break;
-    indexes=GetAuthenticIndexQueue(image);
     for (x=0; x < (ssize_t) image->columns; x++)
     {
       if ((tga_info.image_type == TGARLEColormap) ||
@@ -434,15 +458,20 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
             /*
               Gray scale.
             */
-            index=(IndexPacket) ReadBlobByte(image);
+            if (ReadBlob(image,1,pixels) != 1)
+              ThrowReaderException(CorruptImageError,"UnableToReadImageData");
+            index=(Quantum) pixels[0];
             if (tga_info.colormap_type != 0)
               pixel=image->colormap[(ssize_t) ConstrainColormapIndex(image,
-                1UL*index)];
+                (ssize_t) index,exception)];
             else
               {
-                pixel.red=ScaleCharToQuantum((unsigned char) index);
-                pixel.green=ScaleCharToQuantum((unsigned char) index);
-                pixel.blue=ScaleCharToQuantum((unsigned char) index);
+                pixel.red=(MagickRealType) ScaleCharToQuantum((unsigned char)
+                  index);
+                pixel.green=(MagickRealType) ScaleCharToQuantum((unsigned char)
+                  index);
+                pixel.blue=(MagickRealType) ScaleCharToQuantum((unsigned char)
+                  index);
               }
             break;
           }
@@ -453,22 +482,24 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
               range;
 
             /*
-              5 bits each of RGB;
+              5 bits each of RGB.
             */
             if (ReadBlob(image,2,pixels) != 2)
               ThrowReaderException(CorruptImageError,"UnableToReadImageData");
             j=pixels[0];
             k=pixels[1];
             range=GetQuantumRange(5UL);
-            pixel.red=ScaleAnyToQuantum(1UL*(k & 0x7c) >> 2,range);
-            pixel.green=ScaleAnyToQuantum((1UL*(k & 0x03) << 3)+
-              (1UL*(j & 0xe0) >> 5),range);
-            pixel.blue=ScaleAnyToQuantum(1UL*(j & 0x1f),range);
-            if (image->matte != MagickFalse)
-              pixel.opacity=(k & 0x80) == 0 ? (Quantum) TransparentOpacity :
-                (Quantum) OpaqueOpacity;
+            pixel.red=(MagickRealType) ScaleAnyToQuantum(1UL*(k & 0x7c) >> 2,
+              range);
+            pixel.green=(MagickRealType) ScaleAnyToQuantum((1UL*
+              (k & 0x03) << 3)+(1UL*(j & 0xe0) >> 5),range);
+            pixel.blue=(MagickRealType) ScaleAnyToQuantum(1UL*(j & 0x1f),range);
+            if (image->alpha_trait != UndefinedPixelTrait)
+              pixel.alpha=(MagickRealType) ((k & 0x80) == 0 ? (Quantum)
+                TransparentAlpha : (Quantum) OpaqueAlpha);
             if (image->storage_class == PseudoClass)
-              index=ConstrainColormapIndex(image,((size_t) k << 8)+j);
+              index=(Quantum) ConstrainColormapIndex(image,((ssize_t) (k << 8))+
+                j,exception);
             break;
           }
           case 24:
@@ -478,9 +509,9 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
             */
             if (ReadBlob(image,3,pixels) != 3)
               ThrowReaderException(CorruptImageError,"UnableToReadImageData");
-            pixel.blue=ScaleCharToQuantum(pixels[0]);
-            pixel.green=ScaleCharToQuantum(pixels[1]);
-            pixel.red=ScaleCharToQuantum(pixels[2]);
+            pixel.blue=(MagickRealType) ScaleCharToQuantum(pixels[0]);
+            pixel.green=(MagickRealType) ScaleCharToQuantum(pixels[1]);
+            pixel.red=(MagickRealType) ScaleCharToQuantum(pixels[2]);
             break;
           }
           case 32:
@@ -490,33 +521,29 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
             */
             if (ReadBlob(image,4,pixels) != 4)
               ThrowReaderException(CorruptImageError,"UnableToReadImageData");
-            pixel.blue=ScaleCharToQuantum(pixels[0]);
-            pixel.green=ScaleCharToQuantum(pixels[1]);
-            pixel.red=ScaleCharToQuantum(pixels[2]);
-            pixel.opacity=(Quantum) (QuantumRange-ScaleCharToQuantum(
-              pixels[3]));
+            pixel.blue=(MagickRealType) ScaleCharToQuantum(pixels[0]);
+            pixel.green=(MagickRealType) ScaleCharToQuantum(pixels[1]);
+            pixel.red=(MagickRealType) ScaleCharToQuantum(pixels[2]);
+            pixel.alpha=(MagickRealType) ScaleCharToQuantum(pixels[3]);
             break;
           }
         }
       if (status == MagickFalse)
         ThrowReaderException(CorruptImageError,"UnableToReadImageData");
       if (image->storage_class == PseudoClass)
-        SetPixelIndex(indexes+x,index);
-      SetPixelRed(q,pixel.red);
-      SetPixelGreen(q,pixel.green);
-      SetPixelBlue(q,pixel.blue);
-      if (image->matte != MagickFalse)
-        SetPixelOpacity(q,pixel.opacity);
-      q++;
+        SetPixelIndex(image,index,q);
+      SetPixelRed(image,ClampToQuantum(pixel.red),q);
+      SetPixelGreen(image,ClampToQuantum(pixel.green),q);
+      SetPixelBlue(image,ClampToQuantum(pixel.blue),q);
+      if (image->alpha_trait != UndefinedPixelTrait)
+        SetPixelAlpha(image,ClampToQuantum(pixel.alpha),q);
+      q+=GetPixelChannels(image);
     }
-    if (((tga_info.attributes & 0xc0) >> 6) == 4)
-      offset+=4;
+    if (((unsigned char) (tga_info.attributes & 0xc0) >> 6) == 2)
+      offset+=2;
     else
-      if (((tga_info.attributes & 0xc0) >> 6) == 2)
-        offset+=2;
-      else
-        offset++;
-    if (offset >= image->rows)
+      offset++;
+    if (offset >= (ssize_t) image->rows)
       {
         base++;
         offset=base;
@@ -566,33 +593,29 @@ ModuleExport size_t RegisterTGAImage(void)
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("ICB");
+  entry=AcquireMagickInfo("TGA","ICB","Truevision Targa image");
   entry->decoder=(DecodeImageHandler *) ReadTGAImage;
   entry->encoder=(EncodeImageHandler *) WriteTGAImage;
-  entry->adjoin=MagickFalse;
-  entry->description=ConstantString("Truevision Targa image");
-  entry->module=ConstantString("TGA");
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderAdjoinFlag;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("TGA");
+  entry=AcquireMagickInfo("TGA","TGA","Truevision Targa image");
   entry->decoder=(DecodeImageHandler *) ReadTGAImage;
   entry->encoder=(EncodeImageHandler *) WriteTGAImage;
-  entry->adjoin=MagickFalse;
-  entry->description=ConstantString("Truevision Targa image");
-  entry->module=ConstantString("TGA");
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderAdjoinFlag;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("VDA");
+  entry=AcquireMagickInfo("TGA","VDA","Truevision Targa image");
   entry->decoder=(DecodeImageHandler *) ReadTGAImage;
   entry->encoder=(EncodeImageHandler *) WriteTGAImage;
-  entry->adjoin=MagickFalse;
-  entry->description=ConstantString("Truevision Targa image");
-  entry->module=ConstantString("TGA");
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderAdjoinFlag;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("VST");
+  entry=AcquireMagickInfo("TGA","VST","Truevision Targa image");
   entry->decoder=(DecodeImageHandler *) ReadTGAImage;
   entry->encoder=(EncodeImageHandler *) WriteTGAImage;
-  entry->adjoin=MagickFalse;
-  entry->description=ConstantString("Truevision Targa image");
-  entry->module=ConstantString("TGA");
+  entry->flags|=CoderDecoderSeekableStreamFlag;
+  entry->flags^=CoderAdjoinFlag;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -640,7 +663,8 @@ ModuleExport void UnregisterTGAImage(void)
 %
 %  The format of the WriteTGAImage method is:
 %
-%      MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteTGAImage(const ImageInfo *image_info,
+%        Image *image,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -650,11 +674,10 @@ ModuleExport void UnregisterTGAImage(void)
 %
 */
 static inline void WriteTGAPixel(Image *image,TGAImageType image_type,
-  const IndexPacket *indexes,const PixelPacket *p,const QuantumAny range,
-  const double midpoint)
+  const Quantum *p,const QuantumAny range,const double midpoint)
 {
   if (image_type == TGAColormap || image_type == TGARLEColormap)
-    (void) WriteBlobByte(image,(unsigned char) GetPixelIndex(indexes));
+    (void) WriteBlobByte(image,(unsigned char) ((ssize_t) GetPixelIndex(image,p)));
   else
     {
       if (image_type == TGAMonochrome || image_type == TGARLEMonochrome)
@@ -667,34 +690,40 @@ static inline void WriteTGAPixel(Image *image,TGAImageType image_type,
               green,
               value;
 
-              green=(unsigned char) ScaleQuantumToAny(GetPixelGreen(p),range);
-              value=((unsigned char) ScaleQuantumToAny(GetPixelBlue(p),range)) |
-                ((green & 0x07) << 5);
-              (void) WriteBlobByte(image,value);
-              value=(unsigned char) ((((image->matte != MagickFalse) && 
-                (GetPixelOpacity(p) < midpoint)) ? 0x80 : 0) | ((unsigned char)
-                ScaleQuantumToAny(GetPixelRed(p),range) << 2) |
-                ((green & 0x18) >> 3));
-              (void) WriteBlobByte(image,value);
+            green=(unsigned char) ScaleQuantumToAny(GetPixelGreen(image,p),
+              range);
+            value=((unsigned char) ScaleQuantumToAny(GetPixelBlue(image,p),
+              range)) | ((green & 0x07) << 5);
+            (void) WriteBlobByte(image,value);
+            value=(((image->alpha_trait != UndefinedPixelTrait) &&
+              ((double) GetPixelAlpha(image,p) > midpoint)) ? 0x80 : 0) |
+              ((unsigned char) ScaleQuantumToAny(GetPixelRed(image,p),range) <<
+              2) | ((green & 0x18) >> 3);
+            (void) WriteBlobByte(image,value);
           }
         else
           {
-            (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelBlue(p)));
-            (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelGreen(p)));
-            (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelRed(p)));
-            if (image->matte != MagickFalse)
-              (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelAlpha(p)));
+            (void) WriteBlobByte(image,ScaleQuantumToChar(
+              GetPixelBlue(image,p)));
+            (void) WriteBlobByte(image,ScaleQuantumToChar(
+              GetPixelGreen(image,p)));
+            (void) WriteBlobByte(image,ScaleQuantumToChar(
+              GetPixelRed(image,p)));
+            if (image->alpha_trait != UndefinedPixelTrait)
+              (void) WriteBlobByte(image,ScaleQuantumToChar(
+                GetPixelAlpha(image,p)));
           }
     }
 }
 
-static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image,
+  ExceptionInfo *exception)
 {
   CompressionType
     compression;
 
   const char
-    *value;
+    *comment;
 
   const double
     midpoint = QuantumRange/2.0;
@@ -705,23 +734,25 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
   QuantumAny
     range;
 
-  register const IndexPacket
-    *indexes;
-
-  register const PixelPacket
+  const Quantum
     *p;
 
-  register ssize_t
+  ssize_t
     x;
 
-  register ssize_t
+  ssize_t
     i;
 
-  register unsigned char
+  unsigned char
     *q;
 
+  size_t
+    channels;
+
   ssize_t
+    base,
     count,
+    offset,
     y;
 
   TGAInfo
@@ -731,12 +762,14 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
     Open output image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickCoreSignature);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
   /*
@@ -744,15 +777,15 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
   */
   if ((image->columns > 65535L) || (image->rows > 65535L))
     ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
-  (void) TransformImageColorspace(image,sRGBColorspace);
+  (void) TransformImageColorspace(image,sRGBColorspace,exception);
   compression=image->compression;
   if (image_info->compression != UndefinedCompression)
     compression=image_info->compression;
   range=GetQuantumRange(5UL);
   tga_info.id_length=0;
-  value=GetImageProperty(image,"comment");
-  if (value != (const char *) NULL)
-    tga_info.id_length=(unsigned char) MagickMin(strlen(value),255);
+  comment=GetImageProperty(image,"comment",exception);
+  if (comment != (const char *) NULL)
+    tga_info.id_length=(unsigned char) MagickMin(strlen(comment),255);
   tga_info.colormap_type=0;
   tga_info.colormap_index=0;
   tga_info.colormap_length=0;
@@ -764,10 +797,10 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
   tga_info.bits_per_pixel=8;
   tga_info.attributes=0;
   if ((image_info->type != TrueColorType) &&
-      (image_info->type != TrueColorMatteType) &&
+      (image_info->type != TrueColorAlphaType) &&
       (image_info->type != PaletteType) &&
-      (image->matte == MagickFalse) &&
-      (SetImageGray(image,&image->exception) != MagickFalse))
+      (image->alpha_trait == UndefinedPixelTrait) &&
+      (SetImageGray(image,exception) != MagickFalse))
     tga_info.image_type=compression == RLECompression ? TGARLEMonochrome :
       TGAMonochrome;
   else
@@ -780,13 +813,13 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
         if (image_info->depth == 5)
           {
             tga_info.bits_per_pixel=16;
-            if (image->matte != MagickFalse)
+            if (image->alpha_trait != UndefinedPixelTrait)
               tga_info.attributes=1;  /* # of alpha bits */
           }
         else
           {
             tga_info.bits_per_pixel=24;
-            if (image->matte != MagickFalse)
+            if (image->alpha_trait != UndefinedPixelTrait)
               {
                 tga_info.bits_per_pixel=32;
                 tga_info.attributes=8;  /* # of alpha bits */
@@ -807,6 +840,15 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
         else
           tga_info.colormap_size=24;
       }
+  if ((image->orientation == BottomRightOrientation) ||
+      (image->orientation == TopRightOrientation))
+    tga_info.attributes|=(1UL << 4);
+  if ((image->orientation == TopLeftOrientation) ||
+      (image->orientation == UndefinedOrientation) ||
+      (image->orientation == TopRightOrientation))
+    tga_info.attributes|=(1UL << 5);
+  if ((image->columns > 65535) || (image->rows > 65535))
+    ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
   /*
     Write TGA header.
   */
@@ -823,7 +865,7 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
   (void) WriteBlobByte(image,tga_info.bits_per_pixel);
   (void) WriteBlobByte(image,tga_info.attributes);
   if (tga_info.id_length != 0)
-    (void) WriteBlob(image,tga_info.id_length,(unsigned char *) value);
+    (void) WriteBlob(image,tga_info.id_length,(unsigned char *) comment);
   if (tga_info.colormap_type != 0)
     {
       unsigned char
@@ -834,8 +876,8 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
         Dump colormap to file (blue, green, red byte order).
       */
       targa_colormap=(unsigned char *) AcquireQuantumMemory((size_t)
-        tga_info.colormap_length,(tga_info.colormap_size/8)*sizeof(
-        *targa_colormap));
+        tga_info.colormap_length,(tga_info.colormap_size/8)*
+        sizeof(*targa_colormap));
       if (targa_colormap == (unsigned char *) NULL)
         ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
       q=targa_colormap;
@@ -843,20 +885,20 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
       {
         if (image_info->depth == 5)
           {
-            green=(unsigned char) ScaleQuantumToAny(image->colormap[i].green,
-              range);
-            *q++=((unsigned char) ScaleQuantumToAny(image->colormap[i].blue,
-              range)) | ((green & 0x07) << 5);
-            *q++=(((image->matte != MagickFalse) && (
-              (double) image->colormap[i].opacity < midpoint)) ? 0x80 : 0) |
-              ((unsigned char) ScaleQuantumToAny(image->colormap[i].red,
-              range) << 2) | ((green & 0x18) >> 3);
+            green=(unsigned char) ScaleQuantumToAny(ClampToQuantum(
+              image->colormap[i].green),range);
+            *q++=((unsigned char) ScaleQuantumToAny(ClampToQuantum(
+              image->colormap[i].blue),range)) | ((green & 0x07) << 5);
+            *q++=(((image->alpha_trait != UndefinedPixelTrait) && ((double)
+              ClampToQuantum(image->colormap[i].alpha) > midpoint)) ? 0x80 : 0) |
+              ((unsigned char) ScaleQuantumToAny(ClampToQuantum(
+              image->colormap[i].red),range) << 2) | ((green & 0x18) >> 3);
           }
         else
           {
-            *q++=ScaleQuantumToChar(image->colormap[i].blue);
-            *q++=ScaleQuantumToChar(image->colormap[i].green);
-            *q++=ScaleQuantumToChar(image->colormap[i].red);
+            *q++=ScaleQuantumToChar(ClampToQuantum(image->colormap[i].blue));
+            *q++=ScaleQuantumToChar(ClampToQuantum(image->colormap[i].green));
+            *q++=ScaleQuantumToChar(ClampToQuantum(image->colormap[i].red));
           }
       }
       (void) WriteBlob(image,(size_t) ((tga_info.colormap_size/8)*
@@ -866,12 +908,14 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
   /*
     Convert MIFF to TGA raster pixels.
   */
-  for (y=(ssize_t) (image->rows-1); y >= 0; y--)
+  base=0;
+  offset=0;
+  channels=GetPixelChannels(image);
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
-    if (p == (const PixelPacket *) NULL)
+    p=GetVirtualPixels(image,0,offset,image->columns,1,exception);
+    if (p == (const Quantum *) NULL)
       break;
-    indexes=GetVirtualIndexQueue(image);
     if (compression == RLECompression)
       {
         x=0;
@@ -884,23 +928,29 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
           {
             if (tga_info.image_type == TGARLEColormap)
               {
-                if (GetPixelIndex(indexes+i) != GetPixelIndex(indexes+(i-1)))
+                if (GetPixelIndex(image,p+(i*channels)) !=
+                    GetPixelIndex(image,p+((i-1)*channels)))
                   break;
               }
             else
               if (tga_info.image_type == TGARLEMonochrome)
                 {
-                  if (GetPixelLuma(image,p+i) != GetPixelLuma(image,p+(i-1)))
+                  if (GetPixelLuma(image,p+(i*channels)) !=
+                      GetPixelLuma(image,p+((i-1)*channels)))
                     break;
                 }
               else
                 {
-                  if ((GetPixelBlue(p+i) != GetPixelBlue(p+(i-1))) ||
-                      (GetPixelGreen(p+i) != GetPixelGreen(p+(i-1))) ||
-                      (GetPixelRed(p+i) != GetPixelRed(p+(i-1))))
+                  if ((GetPixelBlue(image,p+(i*channels)) !=
+                       GetPixelBlue(image,p+((i-1)*channels))) ||
+                      (GetPixelGreen(image,p+(i*channels)) !=
+                       GetPixelGreen(image,p+((i-1)*channels))) ||
+                      (GetPixelRed(image,p+(i*channels)) !=
+                       GetPixelRed(image,p+((i-1)*channels))))
                     break;
-                  if ((image->matte != MagickFalse) &&
-                      (GetPixelAlpha(p+i) != GetPixelAlpha(p+(i-1))))
+                  if ((image->alpha_trait != UndefinedPixelTrait) &&
+                      (GetPixelAlpha(image,p+(i*channels)) !=
+                       GetPixelAlpha(image,p+(i-1)*channels)))
                     break;
                 }
             i++;
@@ -908,8 +958,7 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
           if (i < 3)
             {
               count+=i;
-              p+=i;
-              indexes+=i;
+              p+=(i*channels);
             }
           if ((i >= 3) || (count == 128) ||
               ((x + i) == (ssize_t) image->columns))
@@ -919,8 +968,8 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
                   (void) WriteBlobByte(image,(unsigned char) (--count));
                   while (count >= 0)
                   {
-                    WriteTGAPixel(image,tga_info.image_type,indexes-(count+1),
-                      p-(count+1),range,midpoint);
+                    WriteTGAPixel(image,tga_info.image_type,p-((count+1)*
+                      channels),range,midpoint);
                     count--;
                   }
                   count=0;
@@ -929,18 +978,27 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
           if (i >= 3)
             {
               (void) WriteBlobByte(image,(unsigned char) ((i-1) | 0x80));
-              WriteTGAPixel(image,tga_info.image_type,indexes,p,range,midpoint);
-              p+=i;
-              indexes+=i;
+              WriteTGAPixel(image,tga_info.image_type,p,range,midpoint);
+              p+=(i*channels);
             }
           x+=i;
         }
       }
     else
+      for (x=0; x < (ssize_t) image->columns; x++)
       {
-        for (x=0; x < (ssize_t) image->columns; x++)
-          WriteTGAPixel(image,tga_info.image_type,indexes+x,p++,range,midpoint);
+        WriteTGAPixel(image,tga_info.image_type,p,range,midpoint);
+        p+=channels;
       }
+     if (((unsigned char) (tga_info.attributes & 0xc0) >> 6) == 2)
+       offset+=2;
+     else
+       offset++;
+      if (offset >= (ssize_t) image->rows)
+        {
+          base++;
+          offset=base;
+        }
     if (image->previous == (Image *) NULL)
       {
         status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
