@@ -17,7 +17,7 @@
 %                                January 2008                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -129,36 +129,6 @@ typedef struct _XPSInfo
 %
 */
 
-static inline int ProfileInteger(MagickByteBuffer *buffer,short int *hex_digits)
-{
-  int
-    c,
-    l,
-    value;
-
-  register ssize_t
-    i;
-
-  l=0;
-  value=0;
-  for (i=0; i < 2; )
-  {
-    c=ReadMagickByteBuffer(buffer);
-    if ((c == EOF) || ((c == '%') && (l == '%')))
-      {
-        value=(-1);
-        break;
-      }
-    l=c;
-    c&=0xff;
-    if (isxdigit(c) == MagickFalse)
-      continue;
-    value=(int) ((size_t) value << 4)+hex_digits[c];
-    i++;
-  }
-  return(value);
-}
-
 static Image *ReadXPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   char
@@ -200,7 +170,7 @@ static Image *ReadXPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   RectangleInfo
     page;
 
-  register ssize_t
+  ssize_t
     i;
 
   unsigned long
@@ -298,6 +268,8 @@ static Image *ReadXPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   options=AcquireString("");
   (void) FormatLocaleString(density,MagickPathExtent,"%gx%g",resolution.x,
     resolution.y);
+  if (image_info->ping != MagickFalse)
+    (void) FormatLocaleString(density,MagickPathExtent,"2.0x2.0");
   (void) FormatLocaleString(options,MagickPathExtent,"-g%.20gx%.20g ",(double)
     page.width,(double) page.height);
   read_info=CloneImageInfo(image_info);
@@ -415,6 +387,13 @@ static Image *ReadXPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     (void) CopyMagickString(postscript_image->magick,image->magick,
       MagickPathExtent);
     postscript_image->page=page;
+    if (image_info->ping != MagickFalse)
+      {
+        postscript_image->magick_columns*=image->resolution.x/2.0;
+        postscript_image->magick_rows*=image->resolution.y/2.0;
+        postscript_image->columns*=image->resolution.x/2.0;
+        postscript_image->rows*=image->resolution.y/2.0;
+      }
     (void) CloneImageProfiles(postscript_image,image);
     (void) CloneImageProperties(postscript_image,image);
     next=SyncNextImageInList(postscript_image);
